class CommandBar {
    private bool visible = false;
    private string searchQuery = "";
    private array<Meta::Plugin@> filteredPlugins;
    private array<Meta::Plugin@> settingPlugins;
    private array<Meta::PluginSetting@> filteredSettings;
    private int selectedIndex = 0;
    private SettingsWindow@ settingsWindow;
    private bool shouldResetScroll = false;
    private float scrollY = 0;

    CommandBar(SettingsWindow@ settingsWindow) {
        @this.settingsWindow = settingsWindow;
    }

    bool get_Visible() { return visible; }
    void set_Visible(bool value) { 
        visible = value;
        if (visible) {
            searchQuery = "";
            UpdateFilteredPlugins();
            selectedIndex = 0;
            shouldResetScroll = true;
            scrollY = 0;

            settingsWindow.Visible = false;
        }
    }

    void Render() {
        if (!visible) return;

        vec2 screen = vec2(Draw::GetWidth(), Draw::GetHeight());
        RenderOverlay(screen);

        float searchWidth = Math::Min(600.0f, screen.x * 0.5f);
        float searchHeight = 40.0f;
        float itemHeight = 18.0f;
        float maxResultsHeight = Math::Min(500.0f, screen.y * 0.6f);
        float totalItems = float(filteredPlugins.Length + filteredSettings.Length);
        float resultsHeight = Math::Min(maxResultsHeight, Math::Max(itemHeight * totalItems + 12, itemHeight + 12));
        
        vec2 searchPos = vec2((screen.x - searchWidth) / 2, screen.y * 0.2f);
        vec2 resultsPos = searchPos + vec2(0, searchHeight + 8);

        UI::PushStyleColor(UI::Col::WindowBg, vec4(0.12f, 0.12f, 0.12f, 1.0f));
        UI::PushStyleColor(UI::Col::FrameBg, vec4(0.12f, 0.12f, 0.12f, 1.0f));

        RenderResultsWindow(resultsPos, searchWidth, resultsHeight);
        RenderSearchWindow(searchPos, searchWidth, searchHeight);

        UI::PopStyleColor(2);
    }

    private void RenderOverlay(vec2 screen) {
        UI::GetBackgroundDrawList().AddRectFilled(vec4(0, 0, screen.x, screen.y), vec4(0, 0, 0, 0.8f));
    }

    private void RenderResultsWindow(vec2 pos, float width, float height) {
        UI::SetNextWindowPos(int(pos.x), int(pos.y));
        if (UI::Begin("##ResultsWindow", UI::WindowFlags::NoTitleBar | UI::WindowFlags::NoResize | UI::WindowFlags::NoMove)) {
            UI::SetWindowSize(vec2(width, height));

            UI::PushStyleVar(UI::StyleVar::ItemSpacing, vec2(4, 2));
            UI::PushStyleVar(UI::StyleVar::FramePadding, vec2(6, 6));
            
            if (shouldResetScroll) {
                UI::SetScrollY(0);
                shouldResetScroll = false;
            }

            RenderResults();

            UI::PopStyleVar(2);
        }
        UI::End();
    }

    private void RenderResults() {
        if (filteredPlugins.Length == 0 && filteredSettings.Length == 0) {
            RenderEmptyState();
            return;
        }

        RenderPlugins();
        RenderSettings();
    }

    private void RenderEmptyState() {
        UI::Text("No results found");
    }

    private void RenderPlugins() {
        for (uint i = 0; i < filteredPlugins.Length; i++) {
            bool isSelected = selectedIndex == int(i);
            UI::Text(Icons::CodeFork);
            UI::SameLine();
            UI::SetCursorPos(UI::GetCursorPos() + vec2(4, 0));
            if (UI::Selectable(filteredPlugins[i].Name + "##" + i, isSelected)) {
                OpenPluginSettings(filteredPlugins[i]);
            }
        }
    }

    private void RenderSettings() {
        for (uint i = 0; i < filteredSettings.Length; i++) {
            bool isSelected = selectedIndex == int(i + filteredPlugins.Length);
            UI::Text(Icons::Cog);
            UI::SameLine();
            UI::SetCursorPos(UI::GetCursorPos() + vec2(4, 0));
            if (UI::Selectable(GetSettingText(i) + "##s" + i, isSelected)) {
                OpenPluginSettingAndFocus(settingPlugins[i], filteredSettings[i]);
            }
        }
    }

    private string GetSettingText(uint index) {
        string text = settingPlugins[index].Name + " > ";
        if (filteredSettings[index].Category.Length > 0) {
            text += "[" + filteredSettings[index].Category + "] ";
        }
        text += filteredSettings[index].Name;
        return text;
    }

    private void RenderSearchWindow(vec2 pos, float width, float height) {
        UI::SetNextWindowPos(int(pos.x), int(pos.y));
        UI::SetNextWindowSize(int(width), int(height));

        if (UI::Begin("##SearchWindow", UI::WindowFlags::NoTitleBar | UI::WindowFlags::NoResize | UI::WindowFlags::NoMove | UI::WindowFlags::NoScrollbar)) {
            float contentHeight = UI::GetTextLineHeight() + 16;
            float yOffset = (height - contentHeight) / 2;
            UI::SetCursorPos(vec2(12, yOffset + 8));
            
            UI::Text(Icons::Search);
            UI::SameLine();
            
            UI::SetCursorPos(vec2(UI::GetCursorPos().x - 8, yOffset + 4));
            UI::SetNextItemWidth(width - 50);

            if (UI::IsWindowAppearing()) {
                UI::SetKeyboardFocusHere();
            }
            
            bool searchChanged = false;
            searchQuery = UI::InputText("##SearchBox", searchQuery, searchChanged);

            if (searchChanged) {
                UpdateFilteredPlugins();
                selectedIndex = 0;
                shouldResetScroll = true;
            }
        }
        UI::End();
    }

    private void UpdateFilteredPlugins() {
        filteredPlugins.RemoveRange(0, filteredPlugins.Length);
        filteredSettings.RemoveRange(0, filteredSettings.Length);
        settingPlugins.RemoveRange(0, settingPlugins.Length);
        auto plugins = Meta::AllPlugins();
        
        string searchLower = searchQuery.ToLower();
        for (uint i = 0; i < plugins.Length; i++) {
            auto settings = GetVisibleSettings(plugins[i]);
            if (settings.Length > 0) {
                bool pluginMatches = searchLower == "" || plugins[i].Name.ToLower().Contains(searchLower);
                if (pluginMatches) {
                    filteredPlugins.InsertLast(plugins[i]);
                }

                for (uint j = 0; j < settings.Length; j++) {
                    if (searchLower == "" || pluginMatches || settings[j].Name.ToLower().Contains(searchLower) || settings[j].Category.ToLower().Contains(searchLower)) {
                        filteredSettings.InsertLast(settings[j]);
                        settingPlugins.InsertLast(plugins[i]);
                    }
                }
            }
        }
    }

    private void OpenPluginSettings(Meta::Plugin@ plugin) {
        visible = false;
        @settingsWindow.Plugin = plugin;
        @settingsWindow.SettingToFocus = null;
        settingsWindow.Visible = true;
    }

    private void OpenPluginSettingAndFocus(Meta::Plugin@ plugin, Meta::PluginSetting@ setting) {
        visible = false;
        @settingsWindow.Plugin = plugin;
        @settingsWindow.SettingToFocus = setting;
        settingsWindow.Visible = true;
    }

    void HandleShortcut() {
        bool ctrlDown = UI::IsKeyDown(UI::Key::ModCtrl);
        if (UI::IsKeyPressed(ShortcutKey) && (!RequireControl || ctrlDown)) {
            Visible = !visible;            
        }
    }

    void HandleNavigation() {
        if (!visible) return;
        
        if (UI::IsKeyPressed(UI::Key::Escape)) {
            visible = false;
            return;
        }
        
        int totalItems = filteredPlugins.Length + filteredSettings.Length;
        if (UI::IsKeyPressed(UI::Key::DownArrow)) {
            selectedIndex = Math::Min(selectedIndex + 1, totalItems - 1);
        }
        
        if (UI::IsKeyPressed(UI::Key::UpArrow)) {
            selectedIndex = Math::Max(selectedIndex - 1, 0);
        }
        
        if (UI::IsKeyPressed(UI::Key::Enter) && totalItems > 0) {
            if (selectedIndex < filteredPlugins.Length) {
                OpenPluginSettings(filteredPlugins[selectedIndex]);
            } else {
                int settingIndex = selectedIndex - filteredPlugins.Length;
                OpenPluginSettingAndFocus(settingPlugins[settingIndex], filteredSettings[settingIndex]);
            }
        }
    }
} 